package cn.com.qmth.stmms.biz.basic.service.impl;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import cn.com.qmth.stmms.biz.basic.dao.PrivilegeDao;
import cn.com.qmth.stmms.biz.basic.dao.RoleInfoDao;
import cn.com.qmth.stmms.biz.basic.model.Privilege;
import cn.com.qmth.stmms.biz.basic.model.RoleInfo;
import cn.com.qmth.stmms.biz.basic.service.PrivilegeService;
import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.common.enums.PrivilegeType;
import cn.com.qmth.stmms.common.enums.Role;

@Service("privilegeService")
public class PrivilegeServiceImpl extends BaseQueryService<Privilege> implements PrivilegeService {

    private static List<Privilege> privileges = null;

    private static Object lock = new Object();

    @PersistenceContext
    private EntityManager entityManager;

    @Autowired
    private PrivilegeDao privilegeDao;

    @Autowired
    private RoleInfoDao roleInfoDao;

    // @Autowired
    // private SystemAuthService authService;
    //
    // @Autowired
    // private SystemCache authCache;

    @Override
    public List<Privilege> getMenuPrivileges() {
        if (privileges == null) {
            synchronized (lock) {
                if (privileges == null) {
                    privileges = privilegeDao.findByPrivilegeTypeOrderByLevelAscSeqAsc(PrivilegeType.MENU);
                    if (privileges == null) {
                        privileges = new ArrayList<>();
                    }
                }
            }
        }
        return privileges;
    }

    @Transactional
    @Override
    public void initRoleAndPrivilege(Integer schoolId) {
        List<RoleInfo> ris = new ArrayList<>();
        ris.add(new RoleInfo(schoolId, Role.SYS_ADMIN, Role.SYS_ADMIN.getName(), 1));
        ris.add(new RoleInfo(schoolId, Role.SCHOOL_ADMIN, Role.SCHOOL_ADMIN.getName(), 2));
        ris.add(new RoleInfo(schoolId, Role.SCANNER, Role.SCANNER.getName(), 3));
        ris.add(new RoleInfo(schoolId, Role.SUBJECT_HEADER, Role.SUBJECT_HEADER.getName(), 4));
        ris.add(new RoleInfo(schoolId, Role.MARKER, Role.MARKER.getName(), 5));
        ris.add(new RoleInfo(schoolId, Role.SCHOOL_VIEWER, Role.SCHOOL_VIEWER.getName(), 6));
        ris.add(new RoleInfo(schoolId, Role.SCHOOL_DEV, Role.SCHOOL_DEV.getName(), 7));
        ris.add(new RoleInfo(schoolId, Role.INSPECTOR, Role.INSPECTOR.getName(), 8));
        ris.add(new RoleInfo(schoolId, Role.SCAN_ADMIN, Role.SCAN_ADMIN.getName(), 9));
        ris.add(new RoleInfo(schoolId, Role.COLLEGE_ADMIN, Role.COLLEGE_ADMIN.getName(), 10));
        roleInfoDao.save(ris);
        initSchoolAdminPrivilege(schoolId);
        initSubjectHeaderPrivilege(schoolId);
        initCollegeAdminPrivilege(schoolId);
        initInspectorPrivilege(schoolId);
        initSchoolViewerPrivilege(schoolId);

        // SystemAuth systemAuth = authService.findOne();
        // if (systemAuth != null &&
        // SystemAuthType.ONLINE.equals(systemAuth.getType()) &&
        // authCache.isAuth()) {
        // disablePrivilege(schoolId, new String[] { "user_list-help_video",
        // "exam_mark-group-help_video",
        // "exam_inspected_info-list-help_video" });
        // }
    }

    // private void disablePrivilege(Integer schoolId, String[] codes) {
    // StringBuilder sql = new StringBuilder();
    // sql.append("update b_role_privilege ");
    // sql.append(" set enable=0 ");
    // sql.append(" where privilege_code in('" + StringUtils.join(codes, "','")
    // + "')");
    // sql.append(" and school_id=" + schoolId);
    // Query query = entityManager.createNativeQuery(sql.toString());
    // query.executeUpdate();
    // }

    private void initSchoolAdminPrivilege(Integer schoolId) {
        Role role = Role.SCHOOL_ADMIN;
        StringBuilder sql = new StringBuilder();
        sql.append(" INSERT INTO b_role_privilege (`school_id`, `role_code`, `privilege_code`,`enable`) ");
        sql.append(" SELECT s.id,'" + role.name() + "',p.`code`,1 from b_privilege p join b_school s ");
        sql.append(" where s.id=" + schoolId);
        Query query = entityManager.createNativeQuery(sql.toString());
        query.executeUpdate();
    }

    private void initSubjectHeaderPrivilege(Integer schoolId) {
        Role role = Role.SUBJECT_HEADER;
        StringBuilder sql = new StringBuilder();
        sql.append("INSERT INTO b_role_privilege (`school_id`, `role_code`, `privilege_code`,`enable`)");
        sql.append(" SELECT s.id,'" + role.name() + "',p.`code`,1 from b_privilege p join b_school s   ");
        sql.append(
                " where p.`code` in('exam_mark','mark_progress','subject_mark_manage','mark_group','group_query','marker',"
                        + "'marker_query','marker_reset_pass','mark_trial','mark_trial_query','mark_library','mark_library_query'");
        sql.append(
                " ,'mark_library_inspect','mark_library_view','mark_library_reject','mark_arbitrate','mark_arbitrate_query','mark_arbitrate_batch_dis',"
                        + "'mark_arbitrate_dis','mark_quality','subject_arbitrate_report','marker_arbitrate_report','reject_manage','reject_list',"
                        + "'reject_list_query','reject_list_info','reject_list_assigned','reject_report','subject_reject_report','marker_reject_report'");
        sql.append(" ,'problem_history','problem_query','problem_batch_reset','problem_reset'");
        sql.append(
                " ,'inspected','inspected_report','inspected_report_query','paper_inspected','paper_inspected_query','paper_inspected_batch','paper_inspected_cancel'"
                        + ",'paper_inspected_track','paper_inspected_dispose','paper_inspected_round'");
        sql.append(
                " ,'exam_score','exam_score_query','exam_report_subject','report_subject_total','report_subject_total_refresh','report_subject_range','report_subject_college',"
                        + "'report_subject_objective_question','report_subject_subjective_question','report_subject_group')");
        sql.append(" and s.id=" + schoolId);
        Query query = entityManager.createNativeQuery(sql.toString());
        query.executeUpdate();

        sql = new StringBuilder();
        sql.append("INSERT INTO b_role_privilege (`school_id`, `role_code`, `privilege_code`,`enable`)");
        sql.append(" SELECT s.id,'" + role.name() + "',p.`code`,0 from b_privilege p join b_school s   ");
        sql.append(" where p.`code` in('group_add','group_close','group_count','group_paperview',"
                + "'group_view','group_edit','group_reset_edit','group_delete' ,"
                + "'marker_enable','marker_disable','problem_export','exam_score_export','report_subject_total_eport','report_subject_teacher',"
                + "'report_subject_class')");
        sql.append(" and s.id=" + schoolId);
        query = entityManager.createNativeQuery(sql.toString());
        query.executeUpdate();
    }

    private void initCollegeAdminPrivilege(Integer schoolId) {
        Role role = Role.COLLEGE_ADMIN;
        StringBuilder sql = new StringBuilder();
        sql.append("INSERT INTO b_role_privilege (`school_id`, `role_code`, `privilege_code`,`enable`)");
        sql.append(" SELECT s.id,'" + role.name() + "',p.`code`,1 from b_privilege p join b_school s   ");
        sql.append(
                " where p.`code` in('exam_mark','mark_progress','subject_mark_manage','mark_group','group_query','marker',"
                        + "'marker_query','marker_reset_pass','mark_trial','mark_trial_query','mark_library','mark_library_query'");
        sql.append(
                " ,'mark_library_inspect','mark_library_view','mark_library_reject','mark_arbitrate','mark_arbitrate_query','mark_arbitrate_batch_dis',"
                        + "'mark_arbitrate_dis','mark_quality','subject_arbitrate_report','marker_arbitrate_report','reject_manage','reject_list',"
                        + "'reject_list_query','reject_list_info','reject_list_assigned','reject_report','subject_reject_report','marker_reject_report'");
        sql.append(" ,'problem_history','problem_query','problem_batch_reset','problem_reset'");
        sql.append(
                " ,'inspected','inspected_report','inspected_report_query','paper_inspected','paper_inspected_query','paper_inspected_batch','paper_inspected_cancel'"
                        + ",'paper_inspected_track','paper_inspected_dispose','paper_inspected_round'");
        sql.append(
                " ,'exam_score','exam_score_query','exam_score_export','exam_report_subject','report_subject_total','report_subject_total_eport',"
                        + "'report_subject_total_refresh','report_subject_range','report_subject_college',"
                        + "'report_subject_teacher','report_subject_class'"
                        + ",'report_subject_objective_question','report_subject_subjective_question','report_subject_group')");
        sql.append(" and s.id=" + schoolId);
        Query query = entityManager.createNativeQuery(sql.toString());
        query.executeUpdate();

        sql = new StringBuilder();
        sql.append("INSERT INTO b_role_privilege (`school_id`, `role_code`, `privilege_code`,`enable`)");
        sql.append(" SELECT s.id,'" + role.name() + "',p.`code`,0 from b_privilege p join b_school s   ");
        sql.append(" where p.`code` in('group_add','group_close','group_count','group_paperview',"
                + "'group_view','group_edit','group_reset_edit','group_delete', "
                + "'marker_enable','marker_disable','problem_export')");
        sql.append(" and s.id=" + schoolId);
        query = entityManager.createNativeQuery(sql.toString());
        query.executeUpdate();
    }

    private void initInspectorPrivilege(Integer schoolId) {
        Role role = Role.INSPECTOR;
        StringBuilder sql = new StringBuilder();
        sql.append(" INSERT INTO b_role_privilege (`school_id`, `role_code`, `privilege_code`,`enable`)");
        sql.append(" SELECT s.id,'" + role.name() + "',p.`code`,1 from b_privilege p join b_school s   ");
        sql.append(
                " where p.`code` in('inspected','inspected_report','inspected_report_query','paper_inspected','paper_inspected_query',"
                        + "'paper_inspected_batch','paper_inspected_cancel','paper_inspected_track','paper_inspected_dispose','paper_inspected_round')");
        sql.append(" and s.id=" + schoolId);
        Query query = entityManager.createNativeQuery(sql.toString());
        query.executeUpdate();
    }

    private void initSchoolViewerPrivilege(Integer schoolId) {
        Role role = Role.SCHOOL_VIEWER;
        StringBuilder sql = new StringBuilder();
        sql.append(" INSERT INTO b_role_privilege (`school_id`, `role_code`, `privilege_code`,`enable`)");
        sql.append(" SELECT s.id,'" + role.name() + "',p.`code`,1 from b_privilege p join b_school s   ");
        sql.append(
                " where p.`code` in('exam_score','exam_score_query','exam_report_subject','report_subject_total','report_subject_total_refresh','operation_log')");
        sql.append(" and s.id=" + schoolId);
        Query query = entityManager.createNativeQuery(sql.toString());
        query.executeUpdate();
    }

    @Override
    public List<Privilege> getPagePrivilegesByParentCode(String parentCode) {
        return privilegeDao.findByPrivilegeTypeAndParentCodeOrderBySeqAsc(PrivilegeType.PAGE, parentCode);
    }
}
